//
// Copyright 2021, Jesús Pérez Lorenzo
//
use std::collections::HashMap;
use serde::{Serialize,Deserialize};
use std::fmt;

use crate::{
  config::{Config,WebServer},
  module::Module,
	appinfo::AppInfo,
};

#[derive(Clone, Debug, Default, Deserialize)]
pub struct DataCollUsers {
  pub fields: String,
}

#[derive(Clone, Debug)]
pub struct AppEnv {
  pub info: AppInfo,
  pub config: Config,
	pub curr_web: usize,
  pub debug_level: String,
  pub appkey: String,
  pub checked: bool,
  pub collections: HashMap<String, toml::Value>,
  pub modules: HashMap<String, Module>,
  // pub modules: HashMap<String, toml::Value>,
}

impl Default for AppEnv {
  fn default() -> Self {
    Self {
      info: AppInfo::default(),
      config: Config::default(),
			curr_web: 0,
      debug_level: String::from(""),
      appkey: String::from(""),
      checked: true,
      collections: HashMap::new(),
      modules: HashMap::new(),
    }
  }
}
impl AppEnv {
  #[allow(clippy::too_many_arguments)]
  pub async fn new(
    config: Config,
		curr_web: usize,
    debug_level: &str,
		info: AppInfo,
    appkey: &str,
    checked: bool,
    collections: HashMap<String, toml::Value>,
    modules: HashMap<String, Module>,
    //    jwt_sign: Option<JwtSignature>,
  ) -> Self {
    let akey = appkey.to_string();
    Self {
      info,
      config,
			curr_web,
      debug_level: debug_level.to_string(),
      appkey: akey,
      checked,
      collections,
      modules,
      //     jwt_sign,
    }
  }
  pub async fn set_appkey(&mut self, app_key: &str, key: &str) {
    if key.is_empty() {
      self.appkey = app_key.to_string();
    }
  }
  #[must_use]
  pub fn has_appkey(&self) -> bool {
    self.appkey.as_str() == ""
  }
  pub fn get_module(&self, key_module: &str) -> Module {
    if let Some(module) = self.modules.get(key_module) {
      module.to_owned()
    } else {
      Module::default()
    }
  }
	pub fn get_curr_websrvr_config(&self) -> WebServer {
		if let Some(cfg) = self.config.websrvrs.get(self.curr_web) {
		  cfg.to_owned()
		} else {
			WebServer::default()
		}
	}
}

#[derive(Eq, PartialEq, Copy, Clone, Debug, Serialize,  Deserialize)]
#[allow(non_camel_case_types)]
pub enum Language {
  none,
  en,
  es,
}
impl Language {
	pub fn set_provider(lang: String) -> Language {
		match lang.as_str() {	
			"none" => Language::none,
			"en" => Language::en,
			"es" => Language::es,
			_ => Language::none,
		}
	}
}
impl Default for Language {
	fn default() -> Self {
		Language::none
	}
}
impl fmt::Display for Language {
	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
		match self {
			Language::none => write!(f,"none"),
			Language::en => write!(f,"en"),
			Language::es => write!(f,"es"),
		}
	}
}
#[derive(Eq, PartialEq, Copy, Clone, Debug, Serialize,  Deserialize)]
#[allow(non_camel_case_types)]
pub enum Policy {
  notset,
  allow,
  deny,
	temporal,
	ondemand,
}
impl Policy {
	pub fn set_provider(lang: String) -> Policy {
		match lang.as_str() {	
			"notset" => Policy::notset,
			"allow" => Policy::allow,
			"deny" => Policy::deny,
			"temporal" => Policy::temporal,
			"ondemand" => Policy::ondemand,
			_ => Policy::notset,
		}
	}
}
impl Default for Policy {
	fn default() -> Self {
		Policy::notset
	}
}
impl fmt::Display for Policy {
	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
		match self {
			Policy::notset => write!(f,"notset"),
			Policy::allow => write!(f,"allow"),
			Policy::deny => write!(f,"deny"),
			Policy::temporal => write!(f,"temporal"),
			Policy::ondemand => write!(f,"ondemand"),
		}
	}
}
#[derive(Eq, PartialEq, Copy, Clone, Debug, Serialize,  Deserialize)]
#[allow(non_camel_case_types)]
pub enum Rol {
  notset,
  private,
  public,
  user,
  group,
  admin,
  devel,
}
impl Rol {
	pub fn set_provider(lang: String) -> Rol {
		match lang.as_str() {	
			"notset" => Rol::notset,
			"pivate" => Rol::private,
			"public" => Rol::public,
			"user" => Rol::user,
			"group" => Rol::group,
			"admin" => Rol::admin,
			"devel" => Rol::devel,
			_ => Rol::notset,
		}
	}
}
impl Default for Rol {
	fn default() -> Self {
		Rol::notset
	}
}
impl fmt::Display for Rol {
	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
		match self {
			Rol::notset => write!(f,"notset"),
			Rol::private => write!(f,"private"),
			Rol::public => write!(f,"public"),
			Rol::user => write!(f,"user"),
			Rol::group => write!(f,"group"),
			Rol::admin => write!(f,"admin"),
			Rol::devel => write!(f,"devel"),
		}
	}
}